@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with gedit; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA  02110-1301  USA
  */
 
@@ -27,11 +27,7 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GUCHARMAP_2
 #include <gucharmap/gucharmap.h>
-#else
-#include <gucharmap/gucharmap-table.h>
-#endif
 
 G_BEGIN_DECLS
 
@@ -56,7 +52,7 @@ typedef struct _GeditCharmapPanel		GeditCharmapPanel;
 struct _GeditCharmapPanel
 {
 	GtkVBox parent_instance;
-	
+
 	/*< private > */
 	GeditCharmapPanelPrivate *priv;
 };
@@ -75,14 +71,11 @@ struct _GeditCharmapPanelClass
  * Public methods
  */
 GType		 gedit_charmap_panel_get_type	   (void) G_GNUC_CONST;
-GType		 gedit_charmap_panel_register_type (GTypeModule * module);
 GtkWidget	*gedit_charmap_panel_new	   (void);
 
-#ifdef HAVE_GUCHARMAP_2
 GucharmapChartable *gedit_charmap_panel_get_chartable (GeditCharmapPanel *panel);
-#else
-GucharmapTable *gedit_charmap_panel_get_table	   (GeditCharmapPanel *panel);
-#endif
+
+void		_gedit_charmap_panel_register_type (GTypeModule *type_module);
 
 G_END_DECLS
 
