@@ -28,72 +28,126 @@
 #include <glib/gi18n-lib.h>
 #include <gedit/gedit-debug.h>
 #include <gedit/gedit-window.h>
+#include <gedit/gedit-window-activatable.h>
 #include <gedit/gedit-panel.h>
 #include <gedit/gedit-document.h>
-#include <gedit/gedit-prefs-manager.h>
 
 #include <gucharmap/gucharmap.h>
 
-#define WINDOW_DATA_KEY	"GeditCharmapPluginWindowData"
-
 #define GEDIT_CHARMAP_PLUGIN_GET_PRIVATE(object) \
 				(G_TYPE_INSTANCE_GET_PRIVATE ((object),	\
 				GEDIT_TYPE_CHARMAP_PLUGIN,		\
 				GeditCharmapPluginPrivate))
 
-typedef struct
+struct _GeditCharmapPluginPrivate
 {
-	GtkWidget	*panel;
-	guint		 context_id;
-} WindowData;
+	GeditWindow     *window;
 
-GEDIT_PLUGIN_REGISTER_TYPE_WITH_CODE (GeditCharmapPlugin, gedit_charmap_plugin,
-		gedit_charmap_panel_register_type (module);
+	GtkWidget       *panel;
+	guint            context_id;
+};
+
+enum
+{
+	PROP_0,
+	PROP_WINDOW
+};
+
+static void gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface);
+
+G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditCharmapPlugin,
+				gedit_charmap_plugin,
+				PEAS_TYPE_EXTENSION_BASE,
+				0,
+				G_IMPLEMENT_INTERFACE_DYNAMIC (GEDIT_TYPE_WINDOW_ACTIVATABLE,
+							       gedit_window_activatable_iface_init)	\
+													\
+				_gedit_charmap_panel_register_type (type_module);			\
 )
 
 static void
 gedit_charmap_plugin_init (GeditCharmapPlugin *plugin)
 {
 	gedit_debug_message (DEBUG_PLUGINS, "GeditCharmapPlugin initializing");
+
+	plugin->priv = GEDIT_CHARMAP_PLUGIN_GET_PRIVATE (plugin);
 }
 
 static void
-gedit_charmap_plugin_finalize (GObject *object)
+gedit_charmap_plugin_dispose (GObject *object)
 {
-	gedit_debug_message (DEBUG_PLUGINS, "GeditCharmapPlugin finalizing");
+	GeditCharmapPlugin *plugin = GEDIT_CHARMAP_PLUGIN (object);
 
-	G_OBJECT_CLASS (gedit_charmap_plugin_parent_class)->finalize (object);
+	gedit_debug_message (DEBUG_PLUGINS, "GeditCharmapPlugin disposing");
+
+	if (plugin->priv->window != NULL)
+	{
+		g_object_unref (plugin->priv->window);
+		plugin->priv->window = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_charmap_plugin_parent_class)->dispose (object);
 }
 
 static void
-free_window_data (WindowData *data)
+gedit_charmap_plugin_set_property (GObject      *object,
+                                   guint         prop_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
 {
-	g_slice_free (WindowData, data);
+	GeditCharmapPlugin *plugin = GEDIT_CHARMAP_PLUGIN (object);
+
+	switch (prop_id)
+	{
+		case PROP_WINDOW:
+			plugin->priv->window = GEDIT_WINDOW (g_value_dup_object (value));
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
+}
+
+static void
+gedit_charmap_plugin_get_property (GObject    *object,
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
+{
+	GeditCharmapPlugin *plugin = GEDIT_CHARMAP_PLUGIN (object);
+
+	switch (prop_id)
+	{
+		case PROP_WINDOW:
+			g_value_set_object (value, plugin->priv->window);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
 }
 
 static void
 on_table_status_message (GucharmapChartable *chartable,
-			 const gchar    *message,
-			 GeditWindow    *window)
+			 const gchar        *message,
+			 GeditCharmapPlugin *plugin)
 {
 	GtkStatusbar *statusbar;
-	WindowData *data;
 
-	statusbar = GTK_STATUSBAR (gedit_window_get_statusbar (window));
-	data = (WindowData *) g_object_get_data (G_OBJECT (window),
-						 WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
+	statusbar = GTK_STATUSBAR (gedit_window_get_statusbar (plugin->priv->window));
 
-	gtk_statusbar_pop (statusbar, data->context_id);
+	gtk_statusbar_pop (statusbar, plugin->priv->context_id);
 
 	if (message)
-		gtk_statusbar_push (statusbar, data->context_id, message);
+		gtk_statusbar_push (statusbar, plugin->priv->context_id, message);
 }
 
 static void
 on_table_sync_active_char (GucharmapChartable *chartable,
 			   GParamSpec         *psepc,
-			   GeditWindow        *window)
+			   GeditCharmapPlugin *plugin)
 {
 	GString *gs;
 	const gchar **temps;
@@ -124,41 +178,36 @@ on_table_sync_active_char (GucharmapChartable *chartable,
 		g_free (temps);
 	}
 
-	on_table_status_message (chartable, gs->str, window);
+	on_table_status_message (chartable, gs->str, plugin);
 	g_string_free (gs, TRUE);
 }
 
 static gboolean
-on_table_focus_out_event (GtkWidget      *drawing_area,
-			  GdkEventFocus  *event,
-			  GeditWindow    *window)
+on_table_focus_out_event (GtkWidget          *drawing_area,
+			  GdkEventFocus      *event,
+			  GeditCharmapPlugin *plugin)
 {
 	GucharmapChartable *chartable;
-	WindowData *data;
-
-	data = (WindowData *) g_object_get_data (G_OBJECT (window),
-						 WINDOW_DATA_KEY);
-	g_return_val_if_fail (data != NULL, FALSE);
 
 	chartable = gedit_charmap_panel_get_chartable
-					(GEDIT_CHARMAP_PANEL (data->panel));
+					(GEDIT_CHARMAP_PANEL (plugin->priv->panel));
 
-	on_table_status_message (chartable, NULL, window);
+	on_table_status_message (chartable, NULL, plugin);
 	return FALSE;
 }
 
 static void
 on_table_activate (GucharmapChartable *chartable,
-		   GeditWindow *window)
+		   GeditWindow        *window)
 {
 	GtkTextView   *view;
 	GtkTextBuffer *document;
 	GtkTextIter start, end;
 	gchar buffer[6];
 	gchar length;
-        gunichar wc;
+	gunichar wc;
 
-        wc = gucharmap_chartable_get_active_character (chartable);
+	wc = gucharmap_chartable_get_active_character (chartable);
 
 	g_return_if_fail (gucharmap_unichar_validate (wc));
 
@@ -184,10 +233,40 @@ on_table_activate (GucharmapChartable *chartable,
 	gtk_text_buffer_end_user_action (document);
 }
 
-static GtkWidget *
-create_charmap_panel (GeditWindow *window)
+static gchar *
+get_document_font ()
 {
-	GtkWidget      *panel;
+	GSettings *editor;
+	gboolean use_default_font;
+	gchar *font;
+
+	editor = g_settings_new ("org.gnome.gedit.preferences.editor");
+
+	use_default_font = g_settings_get_boolean (editor, "use-default-font");
+
+	if (use_default_font)
+	{
+		GSettings *system;
+
+		system = g_settings_new ("org.gnome.desktop.interface");
+		font = g_settings_get_string (system, "monospace-font-name");
+		g_object_unref (system);
+	}
+	else
+	{
+		font = g_settings_get_string (editor, "editor-font");
+	}
+
+	g_object_unref (editor);
+
+	return font;
+}
+
+static GtkWidget *
+create_charmap_panel (GeditCharmapPlugin *plugin)
+{
+	GSettings *settings;
+	GtkWidget *panel;
 	GucharmapChartable *chartable;
 	PangoFontDescription *font_desc;
 	gchar *font;
@@ -195,7 +274,7 @@ create_charmap_panel (GeditWindow *window)
 	panel = gedit_charmap_panel_new ();
 
 	/* Use the same font as the document */
-	font = gedit_prefs_manager_get_editor_font ();
+	font = get_document_font ();
 
 	chartable = gedit_charmap_panel_get_chartable (GEDIT_CHARMAP_PANEL (panel));
 	font_desc = pango_font_description_from_string (font);
@@ -207,19 +286,19 @@ create_charmap_panel (GeditWindow *window)
 	g_signal_connect (chartable,
 			  "notify::active-character",
 			  G_CALLBACK (on_table_sync_active_char),
-			  window);
+			  plugin);
 	g_signal_connect (chartable,
 			  "focus-out-event",
 			  G_CALLBACK (on_table_focus_out_event),
-			  window);
+			  plugin);
 	g_signal_connect (chartable,
 			  "status-message",
 			  G_CALLBACK (on_table_status_message),
-			  window);
+			  plugin);
 	g_signal_connect (chartable,
 			  "activate",
 			  G_CALLBACK (on_table_activate),
-			  window);
+			  plugin->priv->window);
 
 	gtk_widget_show_all (panel);
 
@@ -227,20 +306,19 @@ create_charmap_panel (GeditWindow *window)
 }
 
 static void
-impl_activate (GeditPlugin *plugin,
-	       GeditWindow *window)
+gedit_charmap_plugin_activate (GeditWindowActivatable *activatable)
 {
+	GeditCharmapPluginPrivate *priv;
 	GeditPanel *panel;
 	GtkWidget *image;
 	GtkIconTheme *theme;
 	GtkStatusbar *statusbar;
-	WindowData *data;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	panel = gedit_window_get_side_panel (window);
+	priv = GEDIT_CHARMAP_PLUGIN (activatable)->priv;
 
-	data = g_slice_new (WindowData);
+	panel = gedit_window_get_side_panel (priv->window);
 
 	theme = gtk_icon_theme_get_default ();
 
@@ -251,57 +329,73 @@ impl_activate (GeditPlugin *plugin,
 		image = gtk_image_new_from_icon_name ("gucharmap",
 						      GTK_ICON_SIZE_MENU);
 
-	data->panel = create_charmap_panel (window);
+	priv->panel = create_charmap_panel (GEDIT_CHARMAP_PLUGIN (activatable));
 
 	gedit_panel_add_item (panel,
-			      data->panel,
+			      priv->panel,
+			      "GeditCharmapPanel",
 			      _("Character Map"),
 			      image);
 
 	gtk_object_sink (GTK_OBJECT (image));
 
-	statusbar = GTK_STATUSBAR (gedit_window_get_statusbar (window));
-	data->context_id = gtk_statusbar_get_context_id (statusbar,
+	statusbar = GTK_STATUSBAR (gedit_window_get_statusbar (priv->window));
+	priv->context_id = gtk_statusbar_get_context_id (statusbar,
 							 "Character Description");
-
-	g_object_set_data_full (G_OBJECT (window),
-				WINDOW_DATA_KEY,
-				data,
-				(GDestroyNotify) free_window_data);
 }
 
 static void
-impl_deactivate	(GeditPlugin *plugin,
-		 GeditWindow *window)
+gedit_charmap_plugin_deactivate (GeditWindowActivatable *activatable)
 {
+	GeditCharmapPluginPrivate *priv;
 	GeditPanel *panel;
-	WindowData *data;
 	GucharmapChartable *chartable;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window),
-						 WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
+	priv = GEDIT_CHARMAP_PLUGIN (activatable)->priv;
 
 	chartable = gedit_charmap_panel_get_chartable
-					(GEDIT_CHARMAP_PANEL (data->panel));
-	on_table_status_message (chartable, NULL, window);
+					(GEDIT_CHARMAP_PANEL (priv->panel));
+	on_table_status_message (chartable, NULL,
+				 GEDIT_CHARMAP_PLUGIN (activatable));
 
-	panel = gedit_window_get_side_panel (window);
-	gedit_panel_remove_item (panel, data->panel);
-
-	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
+	panel = gedit_window_get_side_panel (priv->window);
+	gedit_panel_remove_item (panel, priv->panel);
 }
 
 static void
 gedit_charmap_plugin_class_init (GeditCharmapPluginClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GeditPluginClass *plugin_class = GEDIT_PLUGIN_CLASS (klass);
 
-	object_class->finalize = gedit_charmap_plugin_finalize;
+	object_class->dispose = gedit_charmap_plugin_dispose;
+	object_class->set_property = gedit_charmap_plugin_set_property;
+	object_class->get_property = gedit_charmap_plugin_get_property;
 
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
+	g_object_class_override_property (object_class, PROP_WINDOW, "window");
+
+	g_type_class_add_private (object_class, sizeof (GeditCharmapPluginPrivate));
+}
+
+static void
+gedit_charmap_plugin_class_finalize (GeditCharmapPluginClass *klass)
+{
+}
+
+static void
+gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface)
+{
+	iface->activate = gedit_charmap_plugin_activate;
+	iface->deactivate = gedit_charmap_plugin_deactivate;
+}
+
+G_MODULE_EXPORT void
+peas_register_types (PeasObjectModule *module)
+{
+	gedit_charmap_plugin_register_type (G_TYPE_MODULE (module));
+
+	peas_object_module_register_extension_type (module,
+						    GEDIT_TYPE_WINDOW_ACTIVATABLE,
+						    GEDIT_TYPE_CHARMAP_PLUGIN);
 }
